import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

from validation import validate_sales, validate_tank
from ne.decomp import decompose_sales, decompose_tank, generic_daily_decomp, generic_yearly_decomp, generic_weekly_decomp
from release_one.forecaster import forecast_far, forecast_near


# run long term forecast first
store = 166
tank = 2

# load
dat = pd.read_csv(f'/Users/fisherankney/workspace/forecaster_local/data/sales/KT{store}{tank}_sales.csv',
                  usecols=range(1, 6))
dat.date = pd.to_datetime(dat.date, format='%Y-%m-%d')

# validate
sales_dat = dat.to_dict(orient='records')
sales_dat = validate_sales(sales_dat)

# decompose
sales_decomp = decompose_sales(sales_dat)

# forecast
sales_forecast = forecast_far(sales_dat,
                             yearly_decomp=sales_decomp[0],
                             weekly_decomp=sales_decomp[1],
                             forecast_length=90,
                             output_format='df')
sales_forecast


# short term forecast

store = 166
tank = 1

dat = pd.read_csv(f'/Users/fisherankney/workspace/forecaster_local/data/demo/raw/kt{store}_tank_history.csv')
dat = dat[dat['tank_id'] == 1]
dat = dat.to_dict(orient="records")
dat = validate_tank(dat)

# not enough data
no_data = dat[-2:]
no_data
len(no_data) # under 'no data' threshold (432)

if __name__ == '__main__':
    stf = forecast_near(tank_history=no_data, sales_history=sales_dat,
                        forecast_length=144, output_format='df')

plt.close()
sns.set()
fig = sns.lineplot(x=range(0,432), y=predictions, color='blue', alpha=0.7)
plt.show()




# sufficient data
tank_data = dat
len(tank_data) # above 'no data' threshold

forecast_near(tank_history=tank_data, sales_history=sales_dat,
              forecast_length=144, output_format='df') # should run w/out need of sales dat



import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

from release_one.validation import format_sales
from release_one.decomp import decompose_sales
from release_one.forecaster import forecast_far


# declare store / tank
store = 381
tank = 2


# load sales data & save fuel type
dat = pd.read_csv(f'./data/sales/KT{store}{tank}_sales.csv',
                  usecols=range(1, 6))
tank_type = dat.tank_type.iloc[-1]


# format and split dataset
dat = dat.to_dict(orient="records")
dat = format_sales(dat)
train = dat.iloc[:int(0.85*len(dat))]
test = dat.iloc[int(0.85*len(dat)):]


# decompose sales data into GAM curves
decomp = decompose_sales(train)


# long term forecast
forecast = forecast_far(train,
                        forecast_length=360,
                        yearly_decomp=decomp[0],
                        weekly_decomp=decomp[1])

# show and save figure
plt.close()
sns.set()
fig = sns.lineplot(x=test.ds[:360], y=test.y[:360], color='black', alpha=0.5)
fig = sns.lineplot(x=forecast.ds, y=forecast.yhat, color='blue', alpha=0.7)
fig.set_title(f'KT{store}: Fuel type {tank_type}')
fig.legend(['Actual Sales', 'Forecasted Sales'])
fig.set(ylabel='Daily Sales in Gallons', xlabel='Date')
fig.set_title('')
# plt.savefig(fname=f'./figures/forecast_far/KT{store}{tank}.png')
plt.show()





import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

from release_one.validation import format_sales, format_tank
from release_one.decomp import decompose_sales, decompose_tank
from release_one.forecaster import forecast_far, forecast_near

# conduct long term forecast
store = 390
tank = 2

dat = pd.read_csv(f'/Users/fisherankney/workspace/forecaster_local/data/sales/KT{store}{tank}_sales.csv',
                  usecols=range(1, 6))
dat.date = pd.to_datetime(dat.date, format='%Y-%m-%d')
train_sales = dat.iloc[:3795]
test_sales = dat.iloc[3975:]
train_sales = train_sales.to_dict(orient="records")
train_sales = format_sales(train_sales)
decomp = decompose_sales(train_sales)

ltf = forecast_far(train_sales, yearly_decomp=decomp[0], weekly_decomp=decomp[1],
                 forecast_length=90, holiday_decomp=[], custom_events=[],
                 event_factors=[], daily_lift_est=0, output_format='df')


# conduct short term forecast
store = 390
tank = 1

dat = pd.read_csv(f'/Users/fisherankney/workspace/forecaster_local/data/demo/raw/kt{store}_tank_history.csv')
dat = dat[dat['tank_id'] == 1]
dat = dat.to_dict(orient="records")

# validate data
dat = format_tank(dat)
train_tank = dat[:-144]
test_tank = dat[-144:]
test_tank
stf = forecast_near(train_tank, forecast_length=144, output_format='df')

plt.close()
sns.set()
fig = sns.lineplot(x=ltf.ds, y=ltf.yhat, color='blue', alpha=0.7)
plt.show()

plt.close()
sns.set()
fig = sns.lineplot(x=stf.ds, y=stf.yhat, color='blue', alpha=0.7)
plt.show()

from release_one.decomp import generic_daily_decomp
from release_one.decomp import decompose_tank

tank_decomp = decompose_tank(train_tank)
tank_decomp.daily
predictions = pd.DataFrame(columns=['day', 'day_sum',
                                    'hour', 'hour_base',
                                    'hour_estimate'])
predictions['day'] = np.repeat(ltf.ds[:3], 48)
predictions['day_sum'] = np.repeat(ltf.yhat[:3], 48)
predictions['hour'] = np.repeat(tank_decomp.index, 3)
predictions['hour_base'] = predictions.day_sum*(1/48)
daily_rep = pd.concat([tank_decomp.daily] * 3, ignore_index=True)
predictions.reset_index(inplace=True, drop=True)
predictions['hour_estimate'] = predictions.hour_base * daily_rep
predictions

# compare to
plt.close()
sns.set()
fig = sns.lineplot(x=predictions.index, y=predictions.hour_estimate, color='blue', alpha=0.7)
fig = sns.lineplot(x=range(0,144), y=stf.yhat, color='red', alpha=0.7)
plt.show()

# compare to generic decomposition

from release_one.decomp import generic_daily_decomp

preds = pd.DataFrame(columns=['day', 'day_sum',
                                    'hour', 'hour_base',
                                    'hour_estimate'])
preds['day'] = np.repeat(ltf.ds[:3], 48)
preds['day_sum'] = np.repeat(ltf.yhat[:3], 48)
preds['hour'] = np.repeat(tank_decomp.index, 3)
preds['hour_base'] = predictions.day_sum*(1/48)
daily_rep = pd.concat([generic_daily_decomp] * 3, ignore_index=True)
preds.reset_index(inplace=True, drop=True)
preds['hour_estimate'] = preds.hour_base * daily_rep
preds


# compare to
plt.close()

sns.set(rc={'figure.figsize':(8,6)})
fig = sns.lineplot(x=range(0,144), y=test_tank.y, color='black', alpha = 0.7)
fig = sns.lineplot(x=predictions.index, y=predictions.hour_estimate, color='blue', alpha=0.7)
fig = sns.lineplot(x=preds.index, y=preds.hour_estimate, color='green', alpha=0.7)
fig = sns.lineplot(x=range(0,144), y=stf.yhat, color='red', alpha=0.7)
fig.legend(['actual', '1', '2', '3'])
fig.set(ylabel='change in tank level', xlabel='# half hours from prediction')

plt.savefig(fname=f'../figures/forecast_near_validation/KT{store}{tank}.png')
plt.show()



import numpy as np
import pandas as pd
from fbprophet import Prophet

# set standard events
ny = pd.DataFrame({'holiday': "New Year's Day",
                   'ds': pd.to_datetime(['2020-01-01', '2021-01-01', '2022-01-01',
                                         '2023-01-01', '2024-01-01', '2025-01-01']),
                   'factor': 0.7
                   })

mem = pd.DataFrame({'holiday': "Memorial Day",
                    'ds': pd.to_datetime(['2020-05-25', '2021-05-31', '2022-05-30',
                                          '2023-05-29', '2024-05-27', '2025-05-26']),
                    'factor': 0.9
                    })

ind = pd.DataFrame({'holiday': "Independence Day",
                    'ds': pd.to_datetime(['2020-07-04', '2021-07-04', '2022-07-04',
                                          '2023-07-04', '2024-07-04', '2025-07-04']),
                    'factor': 0.7
                    })

lab = pd.DataFrame({'holiday': "Labor Day",
                    'ds': pd.to_datetime(['2020-09-07', '2021-09-06', '2022-09-05',
                                          '2023-09-04', '2024-09-02', '2025-09-01']),
                    'factor': 0.9
                    })

thanks = pd.DataFrame({'holiday': "Thanksgiving",
                       'ds': pd.to_datetime(['2020-11-26', '2021-11-25', '2022-11-24',
                                             '2023-11-23', '2024-11-28', '2025-11-27']),
                       'factor': 0.9
                       })

xmas = pd.DataFrame({'holiday': "Christmas",
                     'ds': pd.to_datetime(['2020-12-25', '2021-12-25', '2022-12-25',
                                           '2023-12-25', '2024-12-25', '2025-12-25']),
                     'factor': 0.9
                     })

standard_events = pd.concat([ny, mem, ind, lab, thanks, xmas]).reset_index(drop=True)


def add_events(events, names, dates, factors):
    new_events = pd.DataFrame({'holiday': [*names],
                               'ds': pd.to_datetime([*dates]),
                               'factor': [*factors]})
    events = pd.concat([events, new_events], axis=0, ignore_index=True)
    return events


# def drop_events(events, names, dates, factors):
#     drop_events = pd.DataFrame({'holiday': [*names],
#                                 'ds': pd.to_datetime(*dates),
#                                 'factor': [*factors]})
#     events = events.loc[not drop_events]
#     return events


def apply_event_factor(forecast, events, output='dict'):

    if type(forecast) != pd.core.frame.DataFrame:  # no historical sales data
        return 'Error - not enough data to provide forecast'

    if len(forecast.ds) == len(forecast.ds.dt.date.unique()):  # long term forecast
        events = events[['ds', 'factor']]
        events_dict = events.set_index('ds').to_dict(orient='index')

        for row in range(len(forecast)):
            if events_dict.get(forecast.ds[row]):
                forecast.loc[row, 'yhat'] *= events_dict.get(forecast.ds[row]).get('factor')

    else:  # short term forecast
        expanded_ds = pd.Series(name='ds', dtype='datetime64[ns]')
        expanded_factors = pd.Series(name='factors', dtype='float')

        for row in range(len(events)):
            expanded_ds = pd.concat([expanded_ds,
                                     pd.Series(pd.date_range(start=events.ds[row], freq='30min', periods=48))],
                                    ignore_index=True)
            expanded_factors = pd.concat([expanded_factors,
                                          pd.Series(np.repeat(events.factor[row], 48))], ignore_index=True)

        events_hh = pd.DataFrame(columns=['ds', 'factor'])
        events_hh['ds'] = expanded_ds
        events_hh['factor'] = expanded_factors

        events_hh_dict = events_hh.set_index('ds').to_dict(orient='index')

        for row in range(len(forecast)):
            if events_hh_dict.get(forecast.ds[row]):
                forecast.loc[row, 'yhat'] *= events_hh_dict.get(forecast.ds[row]).get('factor')

    if output == 'df':
        return forecast
    else:
        return forecast.to_dict(orient='records')


def remove_event_factor(historical, events):
    if len(historical.ds) == len(historical.ds.dt.date.unique()):  # sales histories
        events = events[['ds', 'factor']]
        events_dict = events.set_index('ds').to_dict(orient='index')

        for row in range(len(historical)):
            if events_dict.get(historical.ds[row]):
                historical.loc[row, 'y'] *= 2 - events_dict.get(historical.ds[row]).get('factor')

    else:  # tank histories
        expanded_ds = pd.Series(name='ds', dtype='datetime64[ns]')
        expanded_factors = pd.Series(name='factors', dtype='float')

        for row in range(len(events)):
            expanded_ds = pd.concat([expanded_ds,
                                     pd.Series(pd.date_range(start=events.ds[row], freq='30min', periods=48))],
                                    ignore_index=True)
            expanded_factors = pd.concat([expanded_factors,
                                          pd.Series(np.repeat(events.factor[row], 48))], ignore_index=True)

        events_hh = pd.DataFrame(columns=['ds', 'factor'])
        events_hh['ds'] = expanded_ds
        events_hh['factor'] = expanded_factors

        events_hh_dict = events_hh.set_index('ds').to_dict(orient='index')

        for row in range(len(historical)):
            if events_hh_dict.get(historical.ds[row]):
                historical.loc[row, 'y'] *= 2 - events_hh_dict.get(historical.ds[row]).get('factor')

    return historical




import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

from release_one.validation import format_sales, format_tank, validate_tank
from release_one.decomp import decompose_sales
from release_one.forecaster import forecast_near, forecast_far
from release_one.events import standard_events, add_events, apply_event_factor


# forecast far w/ events

# load and format sales data
store = 813
tank = 2

# data in db format
dat = pd.read_csv(f'/Users/fisherankney/workspace/forecaster_local/data/sales/KT{store}{tank}_sales.csv',
                  usecols=range(1, 6))
dat = dat.loc[:3797]
tank_type = dat.tank_type.iloc[-1]
sales_dat = dat.to_dict(orient='records')

# create forecast
sales_dat = format_sales(sales_dat)
decomp = decompose_sales(sales_dat)
forecast = forecast_far(sales_dat,
                        forecast_length=45,
                        yearly_decomp=decomp[0],
                        weekly_decomp=decomp[1])

plt.close()
sns.set()
fig = sns.lineplot(x=forecast.ds, y=forecast.yhat, color='blue', alpha=0.7)

# add new events
new_events = add_events(standard_events,
                        names=['special event 1',
                               'special event 2'],
                        dates=['2020-06-25',
                               '2020-07-05'],
                        factors=[5,
                                 0.2])

# apply events
new_forecast = apply_event_factor(forecast, new_events, output='df')


fig = sns.lineplot(x=new_forecast.ds, y=new_forecast.yhat, color='black', alpha=0.7)
fig.legend(['no event adjustments', 'event adjustments'])
fig.set(ylabel='daily sales in gallons', xlabel='date')
plt.savefig(fname=f'./figures/forecast_far/event_adjustment.png')
plt.show()




# forecast near w/ events

# data in db format
store = 813
tank = 1

dat = pd.read_csv(f'/Users/fisherankney/workspace/forecaster_local/data/tank/kt{store}_tank_history.csv')
dat = dat.loc[(dat.tank_id == tank)]
dat = dat.to_dict(orient="records")
tank_dat = format_tank(dat)


# validate tank data & forecast
tank_dat = validate_tank(tank_dat, sales_dat)
forecast = forecast_near(tank_dat, forecast_length=144)

plt.close()
sns.set()
fig2 = sns.lineplot(x=forecast.ds, y=forecast.yhat, color='blue', alpha=0.7)

# create and apply event
new_events = add_events(standard_events,
                        names=['special event 3'],
                        dates=['2020-05-26'],
                        factors=[1.5])

new_forecast = apply_event_factor(forecast, new_events, output='df')

fig2 = sns.lineplot(x=new_forecast.ds, y=new_forecast.yhat, color='black', alpha=0.7)
fig2.legend(['no event adjustments', 'event adjustments'])
fig2.set(ylabel='daily sales in gallons', xlabel='date')
plt.savefig(fname=f'./figures/forecast_near/event_adjustment.png')
plt.show()